package org.dasein.persist.attributes;

import java.util.HashMap;
import java.util.Map;

import org.dasein.attributes.AttributeMap;
import org.dasein.attributes.DataTypeMap;
import org.dasein.persist.Execution;
import org.dasein.persist.PersistenceException;
import org.dasein.persist.Transaction;

public class AttributeDAO {
    static public final String ATTRIBUTES  = "attributes";
    static public final String GROUP       = "group";
    static public final String INDEX       = "index";
    static public final String OWNER_CLASS = "ownerClass";
    static public final String OWNER_ID    = "ownerId";
    static public final String TYPE_CLASS  = "typeClass";
    static public final String TYPE_ID     = "typeId";
    static public final String TYPES       = "types";
    
    static private AttributeDAO factory = new AttributeDAO();

    static public AttributeDAO getInstance() {
        return factory;
    }

    public void saveAttributes(Transaction xaction, Class tc, String tcid, Class oc, String ocid, AttributeMap attrs) throws PersistenceException {
        Map<String,Object> state = new HashMap<String,Object>();

        state.put(TYPE_CLASS, tc.getName());
        state.put(TYPE_ID, tcid);
        state.put(OWNER_CLASS, oc.getName());
        state.put(OWNER_ID, ocid);
        state.put(ATTRIBUTES, attrs);
        state.put(TYPES, loadTypes(xaction, tc, tcid));
        xaction.execute(RemoveAttributes.class, state);
        xaction.execute(CreateAttributes.class, state);
    }

    public void saveTypes(Transaction xaction, Class tc, String tcid, DataTypeMap types) throws PersistenceException {
        Map<String,Object> state = new HashMap<String,Object>();
        
        state.put(TYPE_CLASS, tc.getName());
        state.put(TYPE_ID, tcid);
        state.put(ATTRIBUTES, types);
        xaction.execute(RemoveTypes.class, state);
        xaction.execute(CreateTypes.class, state);
    }
    
    public AttributeMap loadAttributes(Class cls, String oid) throws PersistenceException {
        Transaction xaction = Transaction.getInstance();
        
        try {
            Map<String,Object> state = new HashMap<String,Object>();
    
            state.put(OWNER_ID, oid);
            state.put(OWNER_CLASS, cls.getName());
            state = xaction.execute(LoadAttributes.class, state, Execution.getDataSourceName(cls.getName()));
            xaction.commit();
            return new AttributeMap(state);
        }
        finally {
            xaction.rollback();
        }
    }

    public DataTypeMap loadTypes(Class cls, String oid) throws PersistenceException {
        Transaction xaction = Transaction.getInstance();
        
        try {
            DataTypeMap types = loadTypes(xaction, cls, oid);
            
            xaction.commit();
            return types;
        }
        finally {
            xaction.rollback();
        }
    }

    public DataTypeMap loadTypes(Transaction xaction, Class cls, String oid) throws PersistenceException {
        Map<String,Object> state = new HashMap<String,Object>();
        
        state.put(TYPE_ID, oid);
        state.put(TYPE_CLASS, cls.getName());
        state = xaction.execute(LoadTypes.class, state, Execution.getDataSourceName(cls.getName()));
        return (DataTypeMap)state.get(ATTRIBUTES);        
    }
    
    public void removeAttributes(Transaction xaction, Class cls, String oid) throws PersistenceException {
        Map<String,Object> state = new HashMap<String,Object>();

        state.put(OWNER_ID, oid);
        state.put(OWNER_CLASS, cls.getName());
        xaction.execute(RemoveAttributes.class, state);
    }
    
    public void removeTypes(Transaction xaction, Class cls, String oid) throws PersistenceException {
        Map<String,Object> state = new HashMap<String,Object>();
        
        state.put(TYPE_ID, oid);
        state.put(TYPE_CLASS, cls.getName());
        xaction.execute(RemoveTypes.class, state);
    }
    
    public String findOwnerId(Class cls, String name, String value) throws PersistenceException {    	
    	throw new PersistenceException("NOT IMPLEMENTED");
    }
}
